@@ -35,16 +35,18 @@
  * Silicon Graphics, Inc.
  */
 
+#include "extension_string.h"
+
 typedef struct __GLXconfig __GLXconfig;
 struct __GLXconfig {
     __GLXconfig *next;
     GLuint doubleBufferMode;
     GLuint stereoMode;
 
-    GLint redBits, greenBits, blueBits, alphaBits;	/* bits per comp */
+    GLint redBits, greenBits, blueBits, alphaBits;      /* bits per comp */
     GLuint redMask, greenMask, blueMask, alphaMask;
-    GLint rgbBits;		/* total bits for rgb */
-    GLint indexBits;		/* total bits for colorindex */
+    GLint rgbBits;              /* total bits for rgb */
+    GLint indexBits;            /* total bits for colorindex */
 
     GLint accumRedBits, accumGreenBits, accumBlueBits, accumAlphaBits;
     GLint depthBits;
@@ -54,11 +56,9 @@ struct __GLXconfig {
 
     GLint level;
 
-    GLint pixmapMode;
-
     /* GLX */
     GLint visualID;
-    GLint visualType;     /**< One of the GLX X visual types. (i.e., 
+    GLint visualType;     /**< One of the GLX X visual types. (i.e.,
 			   * \c GLX_TRUE_COLOR, etc.)
 			   */
 
@@ -67,7 +67,7 @@ struct __GLXconfig {
 
     /* EXT_visual_info / GLX 1.2 */
     GLint transparentPixel;
-				/*    colors are floats scaled to ints */
+    /*    colors are floats scaled to ints */
     GLint transparentRed, transparentGreen, transparentBlue, transparentAlpha;
     GLint transparentIndex;
 
@@ -78,15 +78,14 @@ struct __GLXconfig {
     /* SGIX_fbconfig / GLX 1.3 */
     GLint drawableType;
     GLint renderType;
-    GLint xRenderable;
     GLint fbconfigID;
 
     /* SGIX_pbuffer / GLX 1.3 */
     GLint maxPbufferWidth;
     GLint maxPbufferHeight;
     GLint maxPbufferPixels;
-    GLint optimalPbufferWidth;   /* Only for SGIX_pbuffer. */
-    GLint optimalPbufferHeight;  /* Only for SGIX_pbuffer. */
+    GLint optimalPbufferWidth;  /* Only for SGIX_pbuffer. */
+    GLint optimalPbufferHeight; /* Only for SGIX_pbuffer. */
 
     /* SGIX_visual_select_group */
     GLint visualSelectGroup;
@@ -94,14 +93,15 @@ struct __GLXconfig {
     /* OML_swap_method */
     GLint swapMethod;
 
-    GLint screen;
-
     /* EXT_texture_from_pixmap */
     GLint bindToTextureRgb;
     GLint bindToTextureRgba;
     GLint bindToMipmapTexture;
     GLint bindToTextureTargets;
     GLint yInverted;
+
+    /* ARB_framebuffer_sRGB */
+    GLint sRGBCapable;
 };
 
 GLint glxConvertToXVisualType(int visualType);
@@ -113,21 +113,22 @@ GLint glxConvertToXVisualType(int visualType);
 */
 typedef struct __GLXscreen __GLXscreen;
 struct __GLXscreen {
-    void          (*destroy)       (__GLXscreen *screen);
+    void (*destroy) (__GLXscreen * screen);
 
-    __GLXcontext *(*createContext) (__GLXscreen *screen,
-				    __GLXconfig *modes,
-				    __GLXcontext *shareContext);
+    __GLXcontext *(*createContext) (__GLXscreen * screen,
+                                    __GLXconfig * modes,
+                                    __GLXcontext * shareContext,
+                                    unsigned num_attribs,
+                                    const uint32_t *attribs,
+                                    int *error);
 
-    __GLXdrawable *(*createDrawable)(ClientPtr client,
-				     __GLXscreen *context,
-				     DrawablePtr pDraw,
-				     XID drawId,
-				     int type,
-				     XID glxDrawId,
-				     __GLXconfig *modes);
-    int            (*swapInterval)  (__GLXdrawable *drawable,
-				     int interval);
+    __GLXdrawable *(*createDrawable) (ClientPtr client,
+                                      __GLXscreen * context,
+                                      DrawablePtr pDraw,
+                                      XID drawId,
+                                      int type,
+                                      XID glxDrawId, __GLXconfig * modes);
+    int (*swapInterval) (__GLXdrawable * drawable, int interval);
 
     ScreenPtr pScreen;
 
@@ -140,26 +141,14 @@ struct __GLXscreen {
     GLint numVisuals;
 
     char *GLextensions;
-
-    char *GLXvendor;
     char *GLXextensions;
+    char *glvnd;
+    unsigned char glx_enable_bits[__GLX_EXT_BYTES];
 
-    /**
-     * \name GLX version supported by this screen.
-     *
-     * Since the GLX version advertised by the server is for the whole server,
-     * the GLX protocol code uses the minimum version supported on all screens.
-     */
-    /*@{*/
-    unsigned GLXmajor;
-    unsigned GLXminor;
-    /*@}*/
-
-    Bool (*CloseScreen)(int index, ScreenPtr pScreen);
+    Bool (*CloseScreen) (ScreenPtr pScreen);
 };
 
+void __glXScreenInit(__GLXscreen * screen, ScreenPtr pScreen);
+void __glXScreenDestroy(__GLXscreen * screen);
 
-void __glXScreenInit(__GLXscreen *screen, ScreenPtr pScreen);
-void __glXScreenDestroy(__GLXscreen *screen);
-
-#endif /* !__GLX_screens_h__ */
+#endif                          /* !__GLX_screens_h__ */
