@@ -72,7 +72,7 @@ static void deleteCursorHideCountsForScreen(ScreenPtr pScreen);
 #define VERIFY_CURSOR(pCursor, cursor, client, access)			\
     do {								\
 	int err;							\
-	err = dixLookupResourceByType((pointer *) &pCursor, cursor,	\
+	err = dixLookupResourceByType((void **) &pCursor, cursor,	\
 				      RT_CURSOR, client, access);	\
 	if (err != Success) {						\
 	    client->errorValue = cursor;				\
@@ -129,8 +129,7 @@ typedef struct _CursorScreen {
 #define Unwrap(as,s,elt,backup)	(((backup) = (s)->elt), (s)->elt = (as)->elt)
 
 /* The cursor doesn't show up until the first XDefineCursor() */
-static Bool CursorVisible = FALSE;
-
+Bool CursorVisible = FALSE;
 Bool EnableCursor = TRUE;
 
 static Bool
@@ -142,12 +141,7 @@ CursorDisplayCursor(DeviceIntPtr pDev, ScreenPtr pScreen, CursorPtr pCursor)
 
     Unwrap(cs, pScreen, DisplayCursor, backupProc);
 
-    /*
-     * Have to check ConnectionInfo to distinguish client requests from
-     * initial root window setup.  Not a great way to do it, I admit.
-     */
-    if (ConnectionInfo)
-        CursorVisible = EnableCursor;
+    CursorVisible = CursorVisible && EnableCursor;
 
     if (cs->pCursorHideCounts != NULL || !CursorVisible) {
         ret = (*pScreen->DisplayCursor) (pDev, pScreen, NullCursor);
@@ -159,6 +153,7 @@ CursorDisplayCursor(DeviceIntPtr pDev, ScreenPtr pScreen, CursorPtr pCursor)
     if (pCursor != CursorCurrent[pDev->id]) {
         CursorEventPtr e;
 
+        UpdateCurrentTimeIf();
         CursorCurrent[pDev->id] = pCursor;
         for (e = cursorEvents; e; e = e->next) {
             if ((e->eventMask & XFixesDisplayCursorNotifyMask)) {
@@ -201,7 +196,7 @@ static int
 XFixesSelectCursorInput(ClientPtr pClient, WindowPtr pWindow, CARD32 eventMask)
 {
     CursorEventPtr *prev, e;
-    pointer val;
+    void *val;
     int rc;
 
     for (prev = &cursorEvents; (e = *prev); prev = &e->next) {
@@ -234,12 +229,12 @@ XFixesSelectCursorInput(ClientPtr pClient, WindowPtr pWindow, CARD32 eventMask)
                                      DixGetAttrAccess);
         if (rc != Success)
             if (!AddResource(pWindow->drawable.id, CursorWindowType,
-                             (pointer) pWindow)) {
+                             (void *) pWindow)) {
                 free(e);
                 return BadAlloc;
             }
 
-        if (!AddResource(e->clientResource, CursorClientType, (pointer) e))
+        if (!AddResource(e->clientResource, CursorClientType, (void *) e))
             return BadAlloc;
 
         *prev = e;
@@ -312,11 +307,9 @@ CopyCursorToImage(CursorPtr pCursor, CARD32 *image)
     int height = pCursor->bits->height;
     int npixels = width * height;
 
-#ifdef ARGB_CURSOR
     if (pCursor->bits->argb)
         memcpy(image, pCursor->bits->argb, npixels * sizeof(CARD32));
     else
-#endif
     {
         unsigned char *srcLine = pCursor->bits->source;
         unsigned char *mskLine = pCursor->bits->mask;
@@ -574,13 +567,13 @@ SProcXFixesGetCursorImageAndName(ClientPtr client)
  * whether it should be replaced with a reference to pCursor.
  */
 
-typedef Bool (*TestCursorFunc) (CursorPtr pOld, pointer closure);
+typedef Bool (*TestCursorFunc) (CursorPtr pOld, void *closure);
 
 typedef struct {
     RESTYPE type;
     TestCursorFunc testCursor;
     CursorPtr pNew;
-    pointer closure;
+    void *closure;
 } ReplaceCursorLookupRec, *ReplaceCursorLookupPtr;
 
 static const RESTYPE CursorRestypes[] = {
@@ -590,7 +583,7 @@ static const RESTYPE CursorRestypes[] = {
 #define NUM_CURSOR_RESTYPES (sizeof (CursorRestypes) / sizeof (CursorRestypes[0]))
 
 static Bool
-ReplaceCursorLookup(pointer value, XID id, pointer closure)
+ReplaceCursorLookup(void *value, XID id, void *closure)
 {
     ReplaceCursorLookupPtr rcl = (ReplaceCursorLookupPtr) closure;
     WindowPtr pWin;
@@ -632,13 +625,13 @@ ReplaceCursorLookup(pointer value, XID id, pointer closure)
 }
 
 static void
-ReplaceCursor(CursorPtr pCursor, TestCursorFunc testCursor, pointer closure)
+ReplaceCursor(CursorPtr pCursor, TestCursorFunc testCursor, void *closure)
 {
     int clientIndex;
     int resIndex;
     ReplaceCursorLookupRec rcl;
 
-    /* 
+    /*
      * Cursors exist only in the resource database, windows and grabs.
      * All of these are always pointed at by the resource database.  Walk
      * the whole thing looking for cursors
@@ -658,7 +651,7 @@ ReplaceCursor(CursorPtr pCursor, TestCursorFunc testCursor, pointer closure)
              */
             LookupClientResourceComplex(clients[clientIndex],
                                         rcl.type,
-                                        ReplaceCursorLookup, (pointer) &rcl);
+                                        ReplaceCursorLookup, (void *) &rcl);
         }
     }
     /* this "knows" that WindowHasNewCursor doesn't depend on it's argument */
@@ -666,7 +659,7 @@ ReplaceCursor(CursorPtr pCursor, TestCursorFunc testCursor, pointer closure)
 }
 
 static Bool
-TestForCursor(CursorPtr pCursor, pointer closure)
+TestForCursor(CursorPtr pCursor, void *closure)
 {
     return (pCursor == (CursorPtr) closure);
 }
@@ -684,7 +677,7 @@ ProcXFixesChangeCursor(ClientPtr client)
     VERIFY_CURSOR(pDestination, stuff->destination, client,
                   DixWriteAccess | DixSetAttrAccess);
 
-    ReplaceCursor(pSource, TestForCursor, (pointer) pDestination);
+    ReplaceCursor(pSource, TestForCursor, (void *) pDestination);
     return Success;
 }
 
@@ -701,7 +694,7 @@ SProcXFixesChangeCursor(ClientPtr client)
 }
 
 static Bool
-TestForCursorName(CursorPtr pCursor, pointer closure)
+TestForCursorName(CursorPtr pCursor, void *closure)
 {
     Atom *pName = closure;
 
@@ -741,7 +734,7 @@ SProcXFixesChangeCursorByName(ClientPtr client)
 
 /*
  * Routines for manipulating the per-screen hide counts list.
- * This list indicates which clients have requested cursor hiding 
+ * This list indicates which clients have requested cursor hiding
  * for that screen.
  */
 
@@ -778,19 +771,17 @@ createCursorHideCount(ClientPtr pClient, ScreenPtr pScreen)
     pChc->pNext = cs->pCursorHideCounts;
     cs->pCursorHideCounts = pChc;
 
-    /* 
+    /*
      * Create a resource for this element so it can be deleted
      * when the client goes away.
      */
-    if (!AddResource(pChc->resource, CursorHideCountType, (pointer) pChc)) {
-        free(pChc);
+    if (!AddResource(pChc->resource, CursorHideCountType, (void *) pChc))
         return BadAlloc;
-    }
 
     return Success;
 }
 
-/* 
+/*
  * Delete the given hide-counts list element from its screen list.
  */
 static void
@@ -818,7 +809,7 @@ deleteCursorHideCount(CursorHideCountPtr pChcToDel, ScreenPtr pScreen)
     }
 }
 
-/* 
+/*
  * Delete all the hide-counts list elements for this screen.
  */
 static void
@@ -847,16 +838,16 @@ ProcXFixesHideCursor(ClientPtr client)
 
     REQUEST_SIZE_MATCH(xXFixesHideCursorReq);
 
-    ret = dixLookupResourceByType((pointer *) &pWin, stuff->window, RT_WINDOW,
+    ret = dixLookupResourceByType((void **) &pWin, stuff->window, RT_WINDOW,
                                   client, DixGetAttrAccess);
     if (ret != Success) {
         client->errorValue = stuff->window;
         return ret;
     }
 
-    /* 
-     * Has client hidden the cursor before on this screen? 
-     * If so, just increment the count. 
+    /*
+     * Has client hidden the cursor before on this screen?
+     * If so, just increment the count.
      */
 
     pChc = findCursorHideCount(client, pWin->drawable.pScreen);
@@ -865,8 +856,8 @@ ProcXFixesHideCursor(ClientPtr client)
         return Success;
     }
 
-    /* 
-     * This is the first time this client has hid the cursor 
+    /*
+     * This is the first time this client has hid the cursor
      * for this screen.
      */
     ret = XaceHook(XACE_SCREEN_ACCESS, client, pWin->drawable.pScreen,
@@ -911,14 +902,14 @@ ProcXFixesShowCursor(ClientPtr client)
 
     REQUEST_SIZE_MATCH(xXFixesShowCursorReq);
 
-    rc = dixLookupResourceByType((pointer *) &pWin, stuff->window, RT_WINDOW,
+    rc = dixLookupResourceByType((void **) &pWin, stuff->window, RT_WINDOW,
                                  client, DixGetAttrAccess);
     if (rc != Success) {
         client->errorValue = stuff->window;
         return rc;
     }
 
-    /* 
+    /*
      * Has client hidden the cursor on this screen?
      * If not, generate an error.
      */
@@ -952,7 +943,7 @@ SProcXFixesShowCursor(ClientPtr client)
 }
 
 static int
-CursorFreeClient(pointer data, XID id)
+CursorFreeClient(void *data, XID id)
 {
     CursorEventPtr old = (CursorEventPtr) data;
     CursorEventPtr *prev, e;
@@ -968,7 +959,7 @@ CursorFreeClient(pointer data, XID id)
 }
 
 static int
-CursorFreeHideCount(pointer data, XID id)
+CursorFreeHideCount(void *data, XID id)
 {
     CursorHideCountPtr pChc = (CursorHideCountPtr) data;
     ScreenPtr pScreen = pChc->pScreen;
@@ -984,7 +975,7 @@ CursorFreeHideCount(pointer data, XID id)
 }
 
 static int
-CursorFreeWindow(pointer data, XID id)
+CursorFreeWindow(void *data, XID id)
 {
     WindowPtr pWindow = (WindowPtr) data;
     CursorEventPtr e, next;
@@ -1062,6 +1053,8 @@ XFixesCursorInit(void)
 
     if (party_like_its_1989)
         CursorVisible = EnableCursor;
+    else
+        CursorVisible = FALSE;
 
     if (!dixRegisterPrivateKey(&CursorScreenPrivateKeyRec, PRIVATE_SCREEN, 0))
         return FALSE;
